using System;
using System.Collections.Generic;
using System.Text;

namespace FastFourierTransform
{
    public struct complexe
    {
        public float reel;
        public float imag;
    } 

    public class FFT
    {

        #region Mthodes statiques prives

        private static complexe mult(complexe x, complexe y)
        {
            complexe z;

            z.reel = x.reel * y.reel - x.imag * y.imag;
            z.imag = x.reel * y.imag + x.imag * y.reel;

            return z;
        }

        private static complexe add(complexe x, complexe y)
        {
            complexe z;

            z.reel = x.reel + y.reel;
            z.imag = x.imag + y.imag;

            return z;
        }

        private static complexe sub(complexe x, complexe y)
        {
            complexe z;

            z.reel = x.reel - y.reel;
            z.imag = x.imag - y.imag;

            return z;
        }

        #endregion


        #region Mthodes statiques publiques

        public static complexe[] Transform(Int16[] samples, int dim, float SIGN)
        {
            complexe[] mat = new complexe[samples.Length];

            complexe WK, HOLD, Q;
            int LX = dim;
            int FLX = 0;
            int N = (int)((System.Math.Log((double)dim) / System.Math.Log((double)2.0)));
            int II = 0;
            int I, J, K, L, JH;
            int[] M = new int[25];
            int IBLOCK, NBLOCK, LBLOCK, LBHALF, ISTART;
            double FK, V;

            //Cre un vecteur de nombre imaginaire
            for (I = 0; I < samples.Length - 1; I++)
            {
                mat[I].reel = samples[I];
                mat[I].imag = 0;
            }

            for (I = 0; I < N; I++)
            {
                M[I] = (int)System.Math.Pow(2.0, (double)(N - I - 1));
            }

            for (L = 0; L < N; L++)
            {
                NBLOCK = (int)System.Math.Pow(2.0, (double)L);
                LBLOCK = LX / NBLOCK;
                LBHALF = LBLOCK / 2;
                K = 0;

                for (IBLOCK = 0; IBLOCK < NBLOCK; IBLOCK++)
                {
                    FK = K;
                    FLX = LX;

                    V = SIGN * 6.2831853 * FK / FLX;

                    WK.reel = (float)(System.Math.Cos(V));
                    WK.imag = (float)(System.Math.Sin(V));
                    ISTART = LBLOCK * IBLOCK;

                    for (I = 0; I < LBHALF; I++)
                    {
                        J = ISTART + I;
                        JH = J + LBHALF;

                        Q = mult(mat[JH], WK);
                        mat[JH] = sub(mat[J], Q);
                        mat[J] = add(mat[J], Q);
                    }

                    for (I = 1; I < N; I++)
                    {
                        II = I;
                        if (K < M[I])
                        {
                            break;
                        }
                        K = K - M[I];
                    }

                    K = K + M[II];
                }
            }
            K = -1;

            for (J = 0; J < LX; J++)
            {
                if (K >= J)
                {
                    HOLD.reel = mat[J].reel;
                    HOLD.imag = mat[J].imag;

                    mat[J].reel = mat[K + 1].reel;
                    mat[J].imag = mat[K + 1].imag;
                    mat[K + 1].reel = HOLD.reel;
                    mat[K + 1].imag = HOLD.imag;
                }

                for (I = 0; I < N; I++)
                {
                    II = I;
                    if (K + 1 < M[I])
                        break;
                    K = K - M[I];
                }

                K = K + M[II];
            }

            if (SIGN >= 0.0)
            {
                for (I = 0; I < LX; I++)
                {
                    mat[I].reel /= (float)(FLX);
                    mat[I].imag /= (float)(FLX);
                }
            }

            return mat;
        }

        public static complexe[] SwapVecteur(complexe[] mat, int dim)
        {
            complexe[] tmp = new complexe[dim];

            for (int i = 0; i < dim / 2; i++)
                tmp[i] = mat[dim / 2 + i];

            for (int i = dim / 2; i < dim; i++)
                tmp[i] = mat[i - (dim / 2)];

            return tmp;
        }

        public static double[] CreateCaculatedPowerVector(complexe[] vector)
        {
            int size = vector.Length / 2;

            double[] power = new double[size];

            for (int i = 0; i < size; i++)
            {
                power[i] = System.Math.Sqrt((vector[i].reel * vector[i].reel) + (vector[i].imag * vector[i].imag));
            }
            return power;
        }

        #endregion
    }
}
